﻿// BzrSharp
// Copyright (C) 2010  Carlos Flores <cafg10@gmail.com>

// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU Lesser General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.

// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
// GNU Lesser General Public License for more details.

// You should have received a copy of the GNU Lesser General Public License
// along with this program. If not, see <http://www.gnu.org/licenses/>.

using System.Collections.Generic;
using System.IO;
using System.Linq;
using Bazaar.Results;
using StructureMap;
using System;

namespace Bazaar.Commands
{
    public class Move : MultipleFile
    {
        private Uri destination;

        public Move(DirectoryInfo directory, IList<FileSystemInfo> files, FileSystemInfo destination)
            :base(directory, files)
        {
            name = "move";
            this.destination = new Uri(destination.FullName);
        }

        public Move(DirectoryInfo directory, IEnumerable<FileSystemInfo> files, string destination)
            : base(directory, files)
        {
            name = "move";
            this.destination = new Uri(destination);
        }

        public Move(DirectoryInfo directory, IEnumerable<string> files, string destination)
            : base(directory, files)
        {
            name = "move";
            this.destination = new Uri(destination);
        }

        public Move(string directory, IEnumerable<string> files, string destination)
            : base(directory, files)
        {
            name = "move";
            this.destination = new Uri(destination);
        }

        public Move(Uri directory, IEnumerable<Uri> files, string destination)
            : base(directory, files)
        {
            name = "move";
            this.destination = new Uri(destination);
        }

        public Move(Uri directory, IEnumerable<Uri> files, Uri destination)
            : base(directory, files)
        {
            name = "move";
            this.destination = destination;
        }

        public Move(DirectoryInfo directory, IList<string> files, FileSystemInfo destination)
            : base(directory, files)
        {
            name = "move";
            this.destination = new Uri(destination.FullName);
        }

        public Move(string directory, string source, string destination)
            : base(directory)
        {
            files.Add(new Uri(source));
        }

        public override IList<string> Arguments
        {
            get
            {
                IList<string> arguments = base.Arguments;
                arguments.Add(destination.LocalPath);
                return arguments;
            }
        }

        public override IEnumerable<Results.IResult> Run()
        {
            if (ValidArguments)
            {
                var client = ObjectFactory.GetInstance<IClient>();
                response = new Response(client.RunCommand(Arguments.ToArray(), workingDirectory.AbsolutePath));
                return Results;
            }
            else
            {
                string arguments = String.Empty;
                foreach (string argument in Arguments)
                {
                    arguments += argument;
                }
                throw new BazaarException("Unexpected Argument: " + arguments);
            }
        }

        private bool ValidArguments
        {
            get
            {
                if (files.Count > 1)
                {
                    return Directory.Exists(destination.AbsolutePath);
                }
                return true;
            }
        }
    }
}
