﻿// BzrSharp
// Copyright (C) 2010  Carlos Flores <cafg10@gmail.com>

// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU Lesser General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.

// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
// GNU Lesser General Public License for more details.

// You should have received a copy of the GNU Lesser General Public License
// along with this program. If not, see <http://www.gnu.org/licenses/>.

using System;
using System.Collections.Generic;
using System.IO;
using System.Linq;

namespace Bazaar.Commands
{
    /// <summary>
    /// Represents a command that must be
    /// </summary>
    public abstract class MultipleFile : Command
    {
        protected IList<Uri> files = new List<Uri>();

        #region Multiple Type Constructors
        protected MultipleFile(DirectoryInfo directory)
            : base(directory)
        {
        }

        protected MultipleFile(IEnumerable<FileSystemInfo> files)
            : base()
        {

            this.files = (from file in files select new Uri(file.FullName)).ToList();
        }

        protected MultipleFile(DirectoryInfo directory, IEnumerable<FileSystemInfo> files)
            : base(directory)
        {
            this.files = (from file in files select new Uri(file.FullName)).ToList();
        }

        protected MultipleFile(string directory, IEnumerable<FileSystemInfo> files)
            : base(directory)
        {
            this.files = (from file in files select new Uri(file.FullName)).ToList();
        }

        protected MultipleFile(string directory, IEnumerable<string> fileNames)
            : base(directory)
        {
            this.files = (from file in fileNames select new Uri(file)).ToList();
        }

        protected MultipleFile(DirectoryInfo directory, IEnumerable<string> fileNames)
            : base(directory)
        {
            this.files = (from file in fileNames select new Uri(file)).ToList();
        }

        protected MultipleFile(WorkingTree workingTree, IEnumerable<string> fileNames)
            : base(workingTree.Root)
        {
            this.files = (from file in fileNames select new Uri(file)).ToList();
        }

        protected MultipleFile(WorkingTree workingTree, IEnumerable<FileSystemInfo> files)
            : base(workingTree.Root)
        {
            this.files = (from file in files select new Uri(file.FullName)).ToList();
        }

        public MultipleFile(Uri directory, IEnumerable<Uri> files)
            : base(directory)
        {
            this.files = files.ToList();
        }

        public MultipleFile(Uri directory, IEnumerable<string> files)
            : base(directory)
        {
            this.files = (from file in files select new Uri(file)).ToList();
        }

        public MultipleFile(string directory)
            : base(directory)
        {
        }
        #endregion

        public override IList<string> Arguments
        {
            get
            {
                return base.Arguments.Concat(from f in files select f.LocalPath).ToList();
            }
        }
    }
}
