﻿// BzrSharp
// Copyright (C) 2010  Carlos Flores <cafg10@gmail.com>

// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU Lesser General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.

// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
// GNU Lesser General Public License for more details.

// You should have received a copy of the GNU Lesser General Public License
// along with this program. If not, see <http://www.gnu.org/licenses/>.

using System.Collections.Generic;
using System.IO;
using System;

namespace Bazaar.Commands
{
    public abstract class SingleFile : Command
    {
        protected Uri file;

        protected SingleFile(DirectoryInfo directory)
            : base(directory)
        {
        }

        protected SingleFile(FileSystemInfo file)
            : base()
        {
            this.file = new Uri(file.FullName);
        }

        protected SingleFile(string dir, string file)
            : base(dir)
        {
            this.file = new Uri(file);
        }

        protected SingleFile(DirectoryInfo directory, FileSystemInfo file)
            : base(directory)
        {
            this.file = new Uri(file.FullName);
        }

        protected SingleFile(WorkingTree workingTree, FileSystemInfo file)
            : base(workingTree.Root)
        {
            this.file = new Uri(file.FullName);
        }

        protected SingleFile(WorkingTree workingTree, string file)
            : base(workingTree.Root)
        {
            this.file = new Uri(file);
        }

        protected SingleFile(Uri workingTree, Uri file)
            : base(workingTree)
        {
            this.file = file;
        }

        public override IList<string> Arguments
        {
            get
            {
                IList<string> arguments = base.Arguments;

                if (file != null)
                {
                    arguments.Add(file.LocalPath);
                }
                return arguments;
            }
        }
    }
}
