// BzrSharp
// Copyright (C) 2010  Carlos Flores <cafg10@gmail.com>

// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU Lesser General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.

// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
// GNU Lesser General Public License for more details.

// You should have received a copy of the GNU Lesser General Public License
// along with this program. If not, see <http://www.gnu.org/licenses/>.

using System;
using System.Collections.Generic;
using System.IO;
using Bazaar.Results;

namespace Bazaar
{
    /// <summary>
    /// A representation of the Bazaar VCS Working Tree copies
    /// </summary>
    public class WorkingTree
    {
        private DirectoryInfo root;

        private IList<Item> items = new List<Item>();
        private IDictionary<StatusCode, FileStatus> changes = new Dictionary<StatusCode, FileStatus>();

        public WorkingTree(string rootDirectory)
        {
            root = new DirectoryInfo(rootDirectory);
            if (!root.Exists)
            {
                throw new BazaarException("The root directory does not exists");
            }
        }

        public WorkingTree(DirectoryInfo directory)
        {
            root = directory;
        }

        public WorkingTree(Uri uri)
        {
            root = new DirectoryInfo(uri.AbsoluteUri);
        }

        public WorkingTree(string rootDirectory, IDictionary<StatusCode, FileStatus> dict)
            : this(rootDirectory)
        {
            changes = dict;
        }

        public WorkingTree(Uri uri, IDictionary<StatusCode, FileStatus> dict)
            : this(uri)
        {
            changes = dict;
        }

        public DirectoryInfo Root
        {
            get { return root; }
        }

        public IList<Item> Items
        {
            get { return items; }
        }

        public IDictionary<StatusCode, FileStatus> Changes
        {
            get { return changes; }
        }
    }
}
